<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Utils\Crypto;

// Import framework stuff
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;

// Import SPL stuff
use \InvalidArgumentException;
use \LogicException;

/**
 * Crypto utilities class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
final class CryptoUtils extends BaseFrameworkSystem {
	/**
	 * Length of output from hash()
	 */
	private static $hashLength = NULL;

	/**
	 * Private constructor, no instance needed. If PHP would have a static initializer ...
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Hashes a given string with a simple but stronger hash function (no salt)
	 * and hex-encode it.
	 *
	 * @param	$str	The string to be hashed
	 * @return	$hash	The hash from string $str
	 * @throws	InvalidArgumentException	If a parameter is not valid
	 * @throws	LogicException	If proper extension ext-mhash is not loaded
	 */
	public static final function hash (string $str) {
		// Validate parameter/mhash extension
		if (empty($str)) {
			// Throw IAE
			throw new InvalidArgumentException('Parameter "str" is empty');
		} elseif (!extension_loaded('mhash')) {
			// Should be there
			throw new LogicException('Extension ext-mhash not loaded');
		}

		// Hash given string with (better secure) hasher
		$hash = bin2hex(mhash(MHASH_SHA256, $str));

		// Return it
		return $hash;
	}

	/**
	 * "Getter" for length of hash() output. This will be "cached" to speed up
	 * things.
	 *
	 * @return	$length		Length of hash() output
	 */
	public static final function getHashLength () {
		// Is it cashed?
		if (is_null(self::$hashLength)) {
			// No, then hash a string and save its length.
			self::$hashLength = strlen(self::hash('abc123'));
		}

		// Return it
		return self::$hashLength;
	}

}
