<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Connection\Database;

// Import framework stuff
use Org\Mxchange\CoreFramework\Connector\Database\DatabaseConnector;
use Org\Mxchange\CoreFramework\Criteria\Criteria;
use Org\Mxchange\CoreFramework\Criteria\Storing\StoreableCriteria;
use Org\Mxchange\CoreFramework\Database\Backend\DatabaseBackend;
use Org\Mxchange\CoreFramework\Registry\Registerable;
use Org\Mxchange\CoreFramework\Middleware\BaseMiddleware;
use Org\Mxchange\CoreFramework\Middleware\Debug\DebugMiddleware;

// Import SPL stuff
use \InvalidArgumentException;

/**
 * Database selector class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class DatabaseConnection extends BaseMiddleware implements DatabaseConnector, Registerable {
	/**
	 * Array for connection data
	 */
	private $connectData = array(
		'login' => '',
		'pass'  => '',
		'dbase' => '',
		'host'  => ''
	);

	/**
	 * The real database layer
	 */
	private $backendInstance = NULL;

	/**
	 * An instance of this class
	 */
	private static $selfInstance = NULL;

	/**
	 * Protected constructor
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates a new database connection layer
	 *
	 * @param	$debugInstance		An instance of a DebugMiddleware class
	 * @param	$backendInstance	An instance of a DatabaseBackend class
	 * @todo	$debugInstance is currently not used
	 */
	public static final function createDatabaseConnection (DebugMiddleware $debugInstance, DatabaseBackend $backendInstance) {
		// Get instance
		$databaseInstance = new DatabaseConnection();

		// Set database layer
		$databaseInstance->setDatabaseLayer($backendInstance);

		// Set db instance
		self::$selfInstance = $databaseInstance;

		// Return instance
		return $databaseInstance;
	}

	/**
	 * Getter for this class
	 *
	 * @return	$selfInstance	An instance of this class
	 */
	public static final function getSelfInstance () {
		return self::$selfInstance;
	}

	/**
	 * Setter for all database connection data. All these parameters may be
	 * supported by the underlaying backend.
	 *
	 * @param	$login	Login name to database
	 * @param	$pass	Passwort for above login
	 * @param	$dbase	Name of used database
	 * @param	$host	Host to connect to (default: 127.0.0.1)
	 * @return	void
	 */
	public final function setConnectionData (string $login, string $pass, string $dbase, string $host = '127.0.0.1') {
		// Transfer connection data
		$this->connectData['login'] = $login;
		$this->connectData['pass']  = $pass;
		$this->connectData['dbase'] = $dbase;
		$this->connectData['host']  = $host;
	}

	/**
	 * Getter for connection data
	 *
	 * @return	$connectData	Connection data stored with this clas
	 */
	public final function getConnectionData () {
		return $this->connectData;
	}

	/**
	 * Setter for the real database layer
	 * @param	$backendInstance	An instance of a DatabaseBackend class
	 * @return	void
	 */
	public final function setDatabaseLayer (DatabaseBackend $backendInstance) {
		$this->backendInstance = $backendInstance;
	}

	/**
	 * Getter for index key
	 *
	 * @return	$indexKey	Index key
	 */
	public final function getIndexKey () {
		return $this->backendInstance->getIndexKey();
	}

	/**
	 * Runs a 'select' statement on the database layer with given table name
	 * and criteria. If this doesn't fail the result will be returned
	 *
	 * @param	$tableName			Name of the 'table' we shall query
	 * @param	$criteriaInstance	An instance of a Criteria class
	 * @return	$result				The result as an array
	 * @throws	InvalidArgumentException	If a parameter is empty
	 */
	public function doSelectByTableCriteria (string $tableName, Criteria $criteriaInstance) {
		// Validate parameter
		if (empty($tableName)) {
			// Throw IAE
			throw new InvalidArgumentException('Parameter "tableName" is empty');
		}

		// Connect to the database
		$this->backendInstance->connectToDatabase();

		// Get result from query
		$result = $this->backendInstance->querySelect($tableName, $criteriaInstance);

		// Return the result
		return $result;
	}

	/**
	 * Getter for last exception
	 *
	 * @return	$exceptionInstance	Last thrown exception
	 */
	public final function getLastException () {
		$exceptionInstance = $this->backendInstance->getLastException();
		return $exceptionInstance;
	}

	/**
	 * 'Inserts' a data set instance into a local file database folder
	 *
	 * @param	$dataSetInstance	A storeable data set
	 * @return	void
	 */
	public function queryInsertDataSet (StoreableCriteria $dataSetInstance) {
		// Connect to the database
		$this->backendInstance->connectToDatabase();

		// Ask the database layer
		$this->backendInstance->queryInsertDataSet($dataSetInstance);
	}

	/**
	 * 'Updates' a data set instance with a database layer
	 *
	 * @param	$dataSetInstance	A storeable data set
	 * @return	void
	 */
	public function queryUpdateDataSet (StoreableCriteria $dataSetInstance) {
		// Connect to the database
		$this->backendInstance->connectToDatabase();

		// Ask the database layer
		$this->backendInstance->queryUpdateDataSet($dataSetInstance);
	}

	/**
	 * Getter for primary key column of specified table name
	 *
	 * @param	$tableName		Name of table we need the primary key column from
	 * @return	$primaryKey		Primary key column of requested table
	 * @throws	InvalidArgumentException	If a parameter is empty
	 */
	public function getPrimaryKeyOfTable (string $tableName) {
		// Validate parameter
		if (empty($tableName)) {
			// Throw IAE
			throw new InvalidArgumentException('Parameter "tableName" is empty');
		}

		// Connect to the database
		$this->backendInstance->connectToDatabase();

		// Ask the database layer
		$primaryKey = $this->backendInstance->getPrimaryKeyOfTable($tableName);

		// Return the value
		return $primaryKey;
	}

	/**
	 * Removes non-public data from given array.
	 *
	 * @param	$data	An array with possible non-public data that needs to be removed.
	 * @return	$data	A cleaned up array with only public data.
	 */
	public function removeNonPublicDataFromArray (array $data) {
		// Connect to the database
		$this->backendInstance->connectToDatabase();

		// Call database backend
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('DB-CONNECTION[' . $this->__toString() . ']: Calling this->backendInstance->removeNonPublicDataFromArray(data) ...');
		$data = $this->backendInstance->removeNonPublicDataFromArray($data);

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('DB-CONNECTION[' . $this->__toString() . ']: data[]=' . gettype($data));
		return $data;
	}

	/**
	 * Count total table rows
	 *
	 * @param	$tableName	Table name
	 * @return	$count		Total row count
	 * @throws	InvalidArgumentException	If a parameter is empty
	 */
	public function countTotalRows (string $tableName) {
		// Validate parameter
		if (empty($tableName)) {
			// Throw IAE
			throw new InvalidArgumentException('Parameter "tableName" is empty');
		}

		// Connect to the database
		$this->backendInstance->connectToDatabase();

		// Ask the database layer
		$count = $this->backendInstance->countTotalRows($tableName);

		// Return the value
		return $count;
	}

}
